/**********************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.editor;

import java.io.ByteArrayInputStream;

import org.eclipse.core.resources.IFile;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLMessages;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.IManagedForm;

public class SMLIFFormPart implements IFormPart
{
	/**
	 * Set when this part is dirty
	 */
	private boolean dirty;
	
	/**
	 * The editor
	 */
	private SMLIFEditor editor;
	
	public SMLIFFormPart(SMLIFEditor editor)
	{
		this.editor = editor;
	}
	
	/**
	 * @see org.eclipse.ui.forms.IFormPart#commit(boolean)
	 */
	public void commit(boolean onSave)
	{
		if (!onSave)
		{			
			return;
		}
		
		/* Walk through the dirty pile and make the appropriate changes to the underlying model */
		IEditorInput input = editor.getEditorInput();
		if (!(input instanceof IFileEditorInput))
			return;
		IFile inputFile = ((IFileEditorInput)input).getFile();			
		try
		{											
			/* Start reading the file */
			StringBuffer buffer = new StringBuffer();
			SMLCommonUtil.serialzeNode(buffer, editor.getDocument(), 0);
			
			inputFile.setContents(new ByteArrayInputStream(buffer.toString().getBytes("UTF-8")), IFile.FORCE, null);				
			dirty = false;
		} 
		catch (Exception e)
		{
			SMLCommonUtil.openErrorWithDetail(SMLMessages.errorSavingTitle, SMLMessages.errorSaving0, e);
		}  
	}


	public void dispose()
	{
	}

	public void initialize(IManagedForm form)
	{		
	}

	public boolean isDirty()
	{
		return dirty;
	}

	public boolean isStale()
	{
		return false;
	}

	public void refresh()
	{		
	}

	public void setFocus()
	{			
	}

	public boolean setFormInput(Object input)
	{
		return false;
	}
	
	
	public void markDirty()
	{
		this.dirty = true;
	}


}
