/**********************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.export;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.io.PrintStream;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLConstants;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLMessages;
import org.eclipse.cosmos.rm.smlif.internal.export.ui.ModelUnitFile;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.osgi.util.NLS;

public class ExportToSMLIF {

	private StringBuffer smlifFileBuffer = new StringBuffer();
	private String rootFileName = "";
	private String rootFilePath = "";
	private StringBuffer genics = new StringBuffer();
	private StringBuffer phenics = new StringBuffer();
	private StringBuffer ruleBindings = new StringBuffer();
		
	private final String nl = SMLConstants.nl;
	private final String tab = SMLConstants.tab;
	
	/**
	 * Add the folder indicated by <code>smlFolder</code> to the SML-IF file indicated
	 * by <code>smlIFFileName</code>
	 * 
	 * @param smlFolder
	 * @param smlIFFileName
	 * @return
	 */
//	public boolean buildSMLIFFileFromFolder(String smlFolder, String smlIFFileName) throws ExportException {
//		IFile file = deleteFileFromWorkspace(smlIFFileName);
//		
//		File smlifFile = new File(smlIFFileName);
////		System.out.println("read directory " + smlFolder);
//
//		File folder = new File(smlFolder);
//		if (!folder.exists()) {
//			throw new ExportException(NLS.bind(SMLMessages.exportErrorDirectoryNotFound, smlFolder));
//		}
//		if (!folder.canRead()) {
//			throw new ExportException(NLS.bind(SMLMessages.exportErrorDirectoryUnreadable, smlFolder));
//		}
//		if (!folder.isDirectory()) {
//			throw new ExportException(NLS.bind(SMLMessages.exportErrorNotADirectory, smlFolder));
//		}
//
//		rootFileName = folder.getName();
//		String modelName = rootFileName;
//		String baseURI = SMLConstants.BASE_URI_PREFIX + rootFileName;
//
//		IPath path = new Path(folder.getPath());
//
//		rootFilePath = path.toPortableString();
//		//rootFilePath = folder.getPath();
//		//rootFilePath = rootFilePath.substring(0, rootFilePath.lastIndexOf(rootFileName)-1);
//		
//		buildMetadata(modelName, baseURI, "Model description");	  
//		
//		File[] children = folder.listFiles();
//		for (int i = 0; i < children.length; i++) {
//			parseFolder(children[i]);
//		}
//
//		appendGenicsAndPhenics();
//
//		// Create directory structure if necessary
//		createDirectoryStructureFor(smlifFile);
//
//		// Write SML-IF document to system
//		PrintStream writer = null;
//		try {
//			writer = new PrintStream(new FileOutputStream(smlifFile));
//			writer.print(smlif.toString());
//		} catch (IOException exc) {
//			throw new ExportException("Could not write SML-IF document: "+smlifFile, exc);
//		} finally {
//			if (writer != null) {
//				writer.close();
//			}
//		}
//		
//		// Refresh file in workspace, if it exists there
//		try {
//			file = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(new Path(smlIFFileName));
//			if (file != null) {
//				file.refreshLocal(IResource.DEPTH_INFINITE, null);
//			}
//		} catch (Exception exc) {
//			exc.printStackTrace();
//		}
//		
//		return true;
//		
//	}

	private void buildMetadata(String modelName, String baseURI, String modelDescription) {
		smlifFileBuffer.append(SMLConstants.smlifStart)
		.append(nl).append(tab).append(tab)
		.append("<smlif:name>").append(modelName).append("</smlif:name>")
		.append(nl).append(tab).append(tab).append("<smlif:version>v1.0</smlif:version>")
		.append(nl).append(tab).append(tab).append("<smlif:displayName locid=\"")
		.append(baseURI).append("/langName\">").append(modelName).append("</smlif:displayName>")
		.append(nl).append(tab).append(tab).append("<smlif:baseURI>").append(baseURI).append("</smlif:baseURI>")
		.append(nl).append(tab).append(tab).append("<smlif:description locid=\"").append(baseURI).append("/langDescription\">")
		.append(nl).append(tab).append(tab).append(tab).append(modelDescription)
		.append(nl).append(tab).append(tab).append("</smlif:description>")
		.append(nl).append(tab).append("</smlif:identity>").append(nl);
	}

	private void createDirectoryStructureFor(File smlifFile) throws ExportException {
		boolean mkdirsResult = false;
		if (!smlifFile.getParentFile().exists()) {
			mkdirsResult = smlifFile.getParentFile().mkdirs();
			if (!mkdirsResult) {
				throw new ExportException(NLS.bind(SMLMessages.exportErrorCouldNotCreateDirectoryStructure, smlifFile.getName()));
			}
		}
	}

//	/**
//	 * Recursively navigate to write all genic and phenic files to the
//	 * new SML-IF document.  If <code>file</code> is a directory, iterate
//	 * through its children, else add the file's contents directly to the
//	 * SML-IF document buffers.
//	 * 
//	 * @param file
//	 */
//	private void parseFolder(File file) throws ExportException {
//
//		if (!fileExistsAndIsReadable(file)) {
//			return;
//		}
//
//		if (file.isDirectory()) {
//			File[] children = file.listFiles();
//			for (int i = 0; i < children.length; i++) {
//				parseFolder(children[i]);
//			}
//		} else {
//			addFileToSMLIF(file);
//		}
//
//	}

	/**
	 * Answer whether a file exists and is readable
	 * 
	 * @param file
	 * @return whether the file exists and is readable
	 */
	private boolean fileExistsAndIsReadable(File file) throws ExportException {
		String name = file.getName();

		if (!file.exists()) {
			//getLog().log(new Status(Status.INFO, PLUGIN_ID, Status.OK, msg, e));
			throw new ExportException(NLS.bind(SMLMessages.exportErrorFileNotFound, name));
		}
		if (!file.canRead()) {
			//getLog().log(new Status(Status.INFO, PLUGIN_ID, Status.OK, msg, e));
			throw new ExportException(NLS.bind(SMLMessages.exportErrorFileNotReadable, name));
		}
		
		return true;
	}

	/**
	 * Add indicated file to SML-IF document buffers, depending on type of file
	 * 
	 * @param file
	 */
	private void addFileToGenicsAndPhenics(ModelUnitFile file) {
		String name = file.getFileName().trim();
		if (name == null) {
			return;
		}

//		/**
//		 * Ensure that only files identified as genic or phenic will be processed
//		 */
//		IFile iFile = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(new Path(file.getAbsolutePath()));
//		System.out.println("iFile name = "+iFile.getName());
//		int docType = SMLCommonUtil.identifyDocumentType(iFile);
//		System.out.println(" type = "+docType);
//		if (docType == ISMLConstants.TYPE_GENIC) {
		if (file.getType() == ISMLConstants.TYPE_GENIC) {
//			System.out.println(" add to genics " + name);
			if (genics.length() == 0) {
				genics.append(SMLConstants.genicsStart);
			}
			addFileToSection(file, genics);
		} else if (file.getType() == ISMLConstants.TYPE_PHENIC) {
//			System.out.println(" add to phenics " + name);
			if (phenics.length() == 0) {
				phenics.append(SMLConstants.phenicsStart);
			}
			addFileToSection(file, phenics);
		}

	}
	
	protected void addFileToRuleBindings(ModelUnitFile modelUnitFile) {
		if (modelUnitFile.getAliases().size() == 0) {
			return;
		}
		if (modelUnitFile.getRulesForBindings().size() == 0) {
			return;
		}
		if (ruleBindings.length() == 0) {
			ruleBindings.append(SMLConstants.ruleBindingsStart);
		}
		List rulesForBindings = modelUnitFile.getRulesForBindings();
		String firstAlias = (String) modelUnitFile.getAliases().get(0);
		for (Iterator iter = rulesForBindings.iterator(); iter.hasNext();) {
			String rule = (String) iter.next();
			ruleBindings.append(SMLConstants.ruleBindingStart).append(
					firstAlias).append(SMLConstants.ruleBindingMiddle).append(
					rule).append(SMLConstants.ruleBindingEnd);
		}
	}

	/**
	 * Add the file to the indicated section of the document.  The document
	 * sections are maintained as StringBuffer instances.
	 * 
	 * @param file the file being added to the SML-IF document
	 * @param sectionBuffer the string buffer for either the genics or phenics section
	 */
	private void addFileToSection(ModelUnitFile file, StringBuffer sectionBuffer) {
		sectionBuffer.append(getFileAlias(file));
		sectionBuffer.append(getFileContent(file));
		sectionBuffer.append(SMLConstants.docDataEnd);
	}

	/**
	 * Construct alias entity for the indicated file
	 * 
	 * @param file file being added to SML-IF document
	 * @return StringBuffer containing alias markup
	 */
	private StringBuffer getFileAlias(ModelUnitFile file) {
		StringBuffer buffer = new StringBuffer();

		// TODO figure out a way to do this more cleanly using the workspace root getFile() API
		/* The below code makes all aliases relative to the workspace root - probably not valid
		IPath path = Path.fromOSString(file.getPath());
		IPath workspaceLocation = ResourcesPlugin.getWorkspace().getRoot().getLocation();
		String name = path.toPortableString().substring(workspaceLocation.toPortableString().length());
		*/
		
		buffer.append(SMLConstants.docAliasesStart);
		
		if (file.getAliases().isEmpty()) {
			String name = file.getFile().getFullPath().toPortableString();
			
			if (name.startsWith(rootFilePath)) {
				name = name.substring(rootFilePath.length() + 1);
			}
			
			appendAliasToBuffer(name, buffer);
		} else {
			List aliases = file.getAliases();
			for (Iterator iter = aliases.iterator(); iter.hasNext();) {
				String alias = (String) iter.next();
				appendAliasToBuffer(alias, buffer);
			}
		}

		buffer.append(SMLConstants.docAliasesEnd);
		return buffer;
	}

	private void appendAliasToBuffer(String alias, StringBuffer buffer) {
		buffer.append(SMLConstants.docAliasStart).append(alias)
		.append(SMLConstants.docAliasEnd);
	}

	/**
	 * Read genic or phenic file and return contents as a string buffer
	 * 
	 * @param modelUnitFile
	 * @return
	 */
	private StringBuffer getFileContent(ModelUnitFile modelUnitFile) {
		StringBuffer result = new StringBuffer();
		result.append(SMLConstants.nl).append(SMLConstants.tab);
		
		LineNumberReader reader = null;
		try {
			File file = new File(modelUnitFile.getFile().getLocation().toOSString());
			reader = new LineNumberReader(new FileReader(file));

			String line = reader.readLine();
			while (line != null) {
				int idx = line.indexOf("?>");
				if (idx != -1) {
					result.append(line.substring(idx + 2));
				} else {
					result.append(line).append(SMLConstants.nl);
				}

				line = reader.readLine();
			}
		} catch (IOException exc) {
			exc.printStackTrace();
		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch (IOException e) {
					/* Not handled */
				}
			}
		}

		result.append(SMLConstants.tab).append(SMLConstants.nl);

		return result;
	}

	/**
	 * Build an SML-IF file from a collection of individual files representing
	 * genic and phenic model units
	 * 
	 * @param modelUnits collection of genic and phenic files
	 * @param smlIFFileName name of output SML-IF file
	 * @param modelDescription 
	 * @param baseURI 
	 * @param modelName 
	 * @return
	 */
	public boolean exportFromFiles(Iterator modelUnits, String smlIFFileName, String modelName, String baseURI, String modelDescription) throws ExportException {
		File smlifFile = new File(smlIFFileName);
		rootFileName = smlifFile.getName();
		modelName = ("".equals(modelName)) ? rootFileName : modelName;
		//String baseURI = SMLConstants.BASE_URI_PREFIX + rootFileName;

		IPath path = new Path("/");

		rootFilePath = path.toPortableString();
		buildMetadata(modelName, baseURI, modelDescription);

		while (modelUnits.hasNext()) {
			ModelUnitFile modelUnitFile = (ModelUnitFile) modelUnits.next();
			String filePath = modelUnitFile.getFile().getLocation().toOSString();
			//System.out.println("read file " + filePath);
			File file = new File(filePath);
			if (!fileExistsAndIsReadable(file)) {
				continue; // skip over bogus files???
				//return false;
			}
			if (file.isDirectory()) {
				throw new ExportException("Export file name should be a file, not a directory: " + filePath, null);
			}
//			rootFileName = modelUnitFile.getName();
//			rootFilePath = modelUnitFile.getPath();
//			rootFilePath = rootFilePath.substring(0, rootFilePath.lastIndexOf(rootFileName) - 1);
			addFileToRuleBindings(modelUnitFile);
			addFileToGenicsAndPhenics(modelUnitFile);
		}
		appendGenicsAndPhenics();

		deleteFileFromWorkspace(smlIFFileName);
		createDirectoryStructureFor(smlifFile);

		try {
			PrintStream writer = new PrintStream(new FileOutputStream(smlifFile));
			writer.print(smlifFileBuffer.toString());
			writer.close();

		} catch (IOException exc) {
			exc.printStackTrace();
			return false;
		}
		
		try {
			IFile file = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(new Path(smlIFFileName));
			if (file != null) {
				file.refreshLocal(IResource.DEPTH_INFINITE, null);
			}
		} catch (Exception exc) {
			exc.printStackTrace();
		}
		
		return true;
	}

	private IFile deleteFileFromWorkspace(String smlIFFileName) throws ExportException {
		IFile file = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(new Path(smlIFFileName));
		try {
			if (file != null) {
				file.delete(true, null);
			}
		} catch (CoreException e) {
			throw new ExportException("Could not delete workspace file: "+file, e);
		}
		return file;
	}

	/**
	 * Copy genic and phenic buffers to SML-IF buffer before
	 * writing file to system.
	 */
	private void appendGenicsAndPhenics() {
		if (hasRuleBindings()) {
			ruleBindings.append(SMLConstants.ruleBindingsEnd);
			smlifFileBuffer.append(SMLConstants.nl).append(ruleBindings);
		}
		if (hasGenics()) {
			genics.append(SMLConstants.genicsEnd);
			smlifFileBuffer.append(SMLConstants.nl).append(genics);
		}
		if (hasPhenics()) {
			phenics.append(SMLConstants.phenicsEnd);
			smlifFileBuffer.append(SMLConstants.nl).append(phenics);
		}
		smlifFileBuffer.append(SMLConstants.smlifEnd);
	}

	private boolean hasGenics() {
		return genics.length() > 0;
	}

	private boolean hasPhenics() {
		return phenics.length() > 0;
	}

	private boolean hasRuleBindings() {
		return ruleBindings.length() > 0;
	}

}
