/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.export.ui;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

public class ExportResourceTreeContentProvider implements ITreeContentProvider {

	public ExportResourceTreeContentProvider() {
	}

	public Object[] getChildren(Object parentElement) {
		IResource[] kids = new IResource[0];
		if (parentElement instanceof IFile) {
			return kids;
		}
		try {
			kids = ((IContainer) parentElement).members();
		} catch (CoreException e) {
			e.printStackTrace();
		}
		if (kids.length == 0) return kids;
		return buildGoodKids(kids);
	}

	/*
	 *  Find all children that are genic or phenic files
	 */
	protected Object[] buildGoodKids(IResource[] kids) {
		List goodKids = new ArrayList(kids.length);
		for (int i = 0; i < kids.length; i++) {
			IResource resource = kids[i];
			if (resource instanceof IContainer) {
				if (!resource.getName().startsWith(".")) {
					if ((resource instanceof IProject) || hasChildren(resource)) {
						goodKids.add(resource);
					}
				}
			} else {
				int type = SMLCommonUtil.identifyDocumentType((IFile) resource);
				if ((type == ISMLConstants.TYPE_GENIC) || (type == ISMLConstants.TYPE_PHENIC)) {
					goodKids.add(resource);
				}
			}
		}
		return goodKids.toArray();
	}

	public Object getParent(Object element) {
		return ((IResource) element).getParent();
	}

	public boolean hasChildren(Object element) {
		IResource res = (IResource) element;
		return (res.getType() != IResource.FILE) && (getChildren(element).length > 0);
	}

	public Object[] getElements(Object inputElement) {
		return getChildren(inputElement);
	}

	public void dispose() {
	}

	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
	}
	
}
