/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.export.ui;

import java.util.HashMap;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLMessages;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IExportWizard;
import org.eclipse.ui.IWorkbench;

public class ExportToSMLIFWizard extends Wizard implements IExportWizard, ICheckStateListener {
	
	protected ExportToSMLIFWizardPage mainPage;
	protected ExportToSMLIFWizardPage2 page2;
	protected IStructuredSelection selection;
	protected HashMap selectedFiles = new HashMap();

	public ExportToSMLIFWizard() {
		super();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	public boolean performFinish() {
		return mainPage.doExport();
	}
	 
	/* (non-Javadoc)
     * @see org.eclipse.jface.wizard.IWizard#addPages()
     */
    public void addPages() {
        super.addPages(); 
        
		mainPage = new ExportToSMLIFWizardPage(SMLMessages.exportWizardPageTitle, selection);
		mainPage.setDescription(SMLMessages.exportWizardFileInstruction);
        
        addPage(mainPage);        

		page2 = new ExportToSMLIFWizardPage2(SMLMessages.exportWizardPageTitle, selection);
		page2.setDescription(SMLMessages.exportWizardDescriptionPageTwo);
        
        addPage(page2);        

        setWindowTitle(SMLMessages.exportWizardTitle);
    }

	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
	}

	public ExportToSMLIFWizardPage getMainPage() {
		return mainPage;
	}

	public void checkStateChanged(CheckStateChangedEvent event) {
		//if (!(event.getElement() instanceof IFile)) return;
		IResource file = (IResource) event.getElement();
		if (event.getChecked()) {
			addFileToSelected(file);
		} else {
			removeFileFromSelected(file);
		}
		page2.updateDocumentList();
		mainPage.updatePageComplete();
	}

	protected void addFileToSelected(IResource file) {
		if (file instanceof IContainer) {
			Object[] kids = ((ExportResourceTreeContentProvider) mainPage
					.getFileTree().getContentProvider()).getChildren(file);
			for (int i = 0; i < kids.length; i++) {
				addFileToSelected((IResource) kids[i]);
			}
			return;
		}
		primAddFileToSelected(file);
	}

	protected void primAddFileToSelected(IResource file) {
		ModelUnitFile muf = (ModelUnitFile) selectedFiles.get(file);
		if (muf == null) {
			muf = new ModelUnitFile((IFile) file);
		}
		selectedFiles.put(file, muf);
	}

	protected void removeFileFromSelected(IResource file) {
		if (file instanceof IContainer) {
			Object[] kids = ((ExportResourceTreeContentProvider) mainPage
					.getFileTree().getContentProvider()).getChildren(file);
			for (int i = 0; i < kids.length; i++) {
				removeFileFromSelected((IResource) kids[i]);
			}
			return;
		}
		primRemoveFileFromSelected(file);
	}

	private void primRemoveFileFromSelected(IResource file) {
		if (selectedFiles.containsKey(file)) {
			selectedFiles.remove(file);
		}
	}

	public HashMap getSelectedFiles() {
		return selectedFiles;
	}



}
