/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.export.ui;

import java.util.Collection;
import java.util.Iterator;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLMessages;
import org.eclipse.cosmos.rm.smlif.internal.export.ExportException;
import org.eclipse.cosmos.rm.smlif.internal.export.ExportToSMLIF;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ContainerCheckedTreeViewer;
import org.eclipse.ui.dialogs.SaveAsDialog;

public class ExportToSMLIFWizardPage extends WizardPage {
	
	protected Text smlifText;
	protected IStructuredSelection selection;
	protected ContainerCheckedTreeViewer fileTree;

	public ExportToSMLIFWizardPage(String pageName, IStructuredSelection selection) {
		super(pageName);
		setTitle(pageName);
		
		setSelection(selection);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.WizardNewFileCreationPage#createAdvancedControls(org.eclipse.swt.widgets.Composite)
	 */	
	public void createControl(Composite parent) {
		Composite topComponent = new Composite(parent, SWT.NONE);
		GridData topComponentData = new GridData(GridData.GRAB_HORIZONTAL
				| GridData.FILL_HORIZONTAL);
		topComponent.setLayoutData(topComponentData);

		GridLayout topComponentLayout = new GridLayout();
		topComponentLayout.numColumns = 2;
		topComponentLayout.makeColumnsEqualWidth = false;
//		topComponentLayout.marginWidth = 3;
//		topComponentLayout.marginHeight = 3;
		topComponent.setLayout(topComponentLayout);
		
		createSelectedResourcesLabel(topComponent);
		createFileTree(topComponent);
		createSelectionButtons(topComponent);
		Group destGroup = createDestinationGroup(topComponent);
		createDestinationText(destGroup);
		createBrowseButtons(destGroup);
						
		setControl(topComponent);
		updatePageComplete();
		Dialog.applyDialogFont(topComponent);
	}

	protected void createSelectedResourcesLabel(Composite topComponent) {
		Label selectedResourcesLabel = new Label(topComponent, SWT.NONE);
		selectedResourcesLabel.setText("Selected resources:");
		GridData selectedResourcesLabelData = new GridData(GridData.GRAB_HORIZONTAL
				| GridData.FILL_HORIZONTAL);
		selectedResourcesLabelData.grabExcessHorizontalSpace = true;
		selectedResourcesLabelData.horizontalSpan = 2;
		selectedResourcesLabel.setLayoutData(selectedResourcesLabelData);
	}

	protected void createBrowseButtons(Group destGroup) {
		Composite browseButtonRow = new Composite(destGroup, SWT.NONE);
		GridLayout browseButtonRowLayout = new GridLayout();
		browseButtonRowLayout.numColumns = 2;
		browseButtonRowLayout.marginHeight = 0;
		browseButtonRowLayout.marginWidth = 0;
		browseButtonRowLayout.makeColumnsEqualWidth = true;
		GridData gd = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gd.horizontalSpan = GridData.FILL;
		browseButtonRow.setLayoutData(gd);
		browseButtonRow.setLayout(browseButtonRowLayout);
		
		Button browseWorkspaceButton = new Button(browseButtonRow, SWT.PUSH);
		browseWorkspaceButton.setText("Browse Workspace...");
		browseWorkspaceButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				promptForWorkspaceDestinationFile();
			}
		});
		
		Button browseFileSystemButton = new Button(browseButtonRow, SWT.PUSH);
		browseFileSystemButton.setText("Browse File System...");
		browseFileSystemButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				promptForExternalDestinationFile();
			}
		});
	}

	protected void createDestinationText(Group destGroup) {
		smlifText = new Text(destGroup, SWT.BORDER);
		smlifText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				updatePageComplete();
			}
		});
		GridData smlifGridData = new GridData(GridData.FILL_HORIZONTAL);
		smlifGridData.grabExcessHorizontalSpace = true;
		smlifGridData.horizontalSpan = GridData.FILL;
		smlifText.setLayoutData(smlifGridData);
	}

	protected void promptForWorkspaceDestinationFile() {
		SaveAsDialog dialog = new SaveAsDialog(getShell());
		dialog.setTitle("Save SML-IF file as");
		dialog.setOriginalName("smlif.xml");
		//dialog.setFilterPath(smlResourcePath);
		//dialog.s(new String[] { "*.xml", "*" });

		if (dialog.open() == Window.CANCEL) {
			return;
		}
		IPath name = (IPath) dialog.getResult();
		String relativePathToFile = name.toString();
		if (relativePathToFile != null) {
			smlifText.setText(relativePathToFile);
		}
	}

	protected void createSelectionButtons(Composite topComponent) {
		Composite buttonComposite = new Composite(topComponent, SWT.NONE);
		GridLayout buttonLayout = new GridLayout();
		buttonLayout.numColumns = 1;
		buttonComposite.setLayout(buttonLayout);
		GridData buttonCompositeData = new GridData();
		buttonCompositeData.verticalAlignment = GridData.FILL;
		buttonComposite.setLayoutData(buttonCompositeData);
		
		Button selectAllButton = new Button(buttonComposite, SWT.NONE);
		selectAllButton.setText("Select all");
		selectAllButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				getFileTree().setAllChecked(true);
			}
		});
		GridData buttonData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		selectAllButton.setLayoutData(buttonData);

		Button deselectAllButton = new Button(buttonComposite, SWT.NONE);
		deselectAllButton.setText("Deselect all");
		deselectAllButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				getFileTree().setAllChecked(false);
			}
		});
		deselectAllButton.setLayoutData(buttonData);
	}

	protected void createFileTree(Composite topComponent) {
		setFileTree(new ContainerCheckedTreeViewer(topComponent));
		ExportResourceTreeContentProvider contentProvider = new ExportResourceTreeContentProvider();
		getFileTree().setContentProvider(contentProvider);
		getFileTree().setLabelProvider(new ExportResourceTreeLabelProvider());
		getFileTree().setInput(ResourcesPlugin.getWorkspace().getRoot());
		//fileTree.addPostSelectionChangedListener((ExportToSMLIFWizard) getWizard());
		getFileTree().addCheckStateListener((ExportToSMLIFWizard) getWizard());
		
		GridData fileTreeData = new GridData(GridData.FILL_BOTH);
		fileTreeData.verticalSpan = 2;
		fileTreeData.grabExcessVerticalSpace = true;
		getFileTree().getControl().setLayoutData(fileTreeData);
		
		initCheckedItems();
	}
	
	protected void initCheckedItems() {
		for (Iterator iter = selection.iterator(); iter.hasNext();) {
			IResource element = (IResource) iter.next();
			
			getFileTree().setChecked(element, true);
		}
	}

	 public boolean doExport() {
		String smlifFileName = smlifText.getText();
		if (fileIsWorkspacePath(smlifFileName)) {
			// Convert to absolute filesystem path
			IFile file = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(smlifFileName));
			if (file != null) {
				smlifFileName = file.getLocation().toOSString();
			}
		}
		ExportToSMLIF exporter = new ExportToSMLIF();
		boolean result = false;
		try {
			result = exporter.exportFromFiles(getSelectedModelUnits().iterator(), smlifFileName, getPage2().getSMLIFName(), getPage2().getBaseURI(), getPage2().getSMLIFDescription());;
		} catch (ExportException ex) {
			SMLCommonUtil.openErrorWithDetail(SMLMessages.exportErrorTitle, ex.getMessage(), ex);
		}
		return result;
	}

	protected ExportToSMLIFWizardPage2 getPage2() {
		return ((ExportToSMLIFWizard) getWizard()).page2;
	}

	protected boolean fileIsWorkspacePath(String smlifFileName) {
		// Since it's difficult to tell between workspace paths and
		// filesystem paths when on UNIX, we will see if the first
		// path segment (i.e. the project name) exists in the workspace,
		// and answer based on this
		
		String firstSegment = new Path(smlifFileName).segment(0);
		if (firstSegment == null) {
			return false;
		}
		return ResourcesPlugin.getWorkspace().getRoot().exists(new Path(firstSegment));
	}

	protected Collection getSelectedModelUnits() {
		// We have to exclude the folders & projects, and just pass in
		// the files
//		Object[] checkedElements = getFileTree().getCheckedElements();
		return getExportWizard().getSelectedFiles().values();
//		for (int i = 0; i < checkedElements.length; i++) {
//			IResource resource = (IResource) checkedElements[i];
//			if (resource instanceof IFile) {
//				allChecked.add(resource);
//			}
//		}
	}

	private ExportToSMLIFWizard getExportWizard() {
		return (ExportToSMLIFWizard) getWizard();
	}

	public void setSelection(IStructuredSelection selection) {
		this.selection = selection;
	}

	protected Group createDestinationGroup(Composite topComposite) {
		Group destinationGroup = new Group(topComposite, SWT.NONE);
		destinationGroup.setText(SMLMessages.exportWizardDestinationGroupTitle);
		GridData destinationGroupData = new GridData(GridData.GRAB_HORIZONTAL
				| GridData.FILL_HORIZONTAL);
		destinationGroupData.horizontalSpan = 2;
		destinationGroup.setLayoutData(destinationGroupData);
	
		GridLayout destinationGroupLayout = new GridLayout();
		destinationGroupLayout.numColumns = 2;
		destinationGroupLayout.makeColumnsEqualWidth = false;
		destinationGroupLayout.marginWidth = 5;
		destinationGroupLayout.marginHeight = 5;
		//destinationGroupLayout.marginLeft = 3;
		//destinationGroupLayout.marginRight = 3;
		destinationGroup.setLayout(destinationGroupLayout);
		return destinationGroup;
	}

	protected void updatePageComplete() {
		setPageComplete((smlifText.getText().length() > 0) && (getCheckedCount() > 0));
	}

	protected int getCheckedCount() {
		return ((ExportToSMLIFWizard) getWizard()).getSelectedFiles().size();
	}

	protected void promptForExternalDestinationFile() {
		FileDialog dialog = new FileDialog(getShell());
		dialog.setText("Save SML-IF file as");
		dialog.setFileName("smlif.xml");
		//dialog.setFilterPath(smlResourcePath);
		dialog.setFilterExtensions(new String[] { "*.xml", "*" });

		String name = dialog.open();
		if (name != null) {
			smlifText.setText(name);
		}
	}

	public void setFileTree(ContainerCheckedTreeViewer fileTree) {
		this.fileTree = fileTree;
	}

	public ContainerCheckedTreeViewer getFileTree() {
		return fileTree;
	}
}
