/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.export.ui;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.List;

import org.eclipse.cosmos.rm.smlif.internal.common.SMLMessages;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ListDialog;

public class ExportToSMLIFWizardPage2 extends WizardPage {

	protected IStructuredSelection selection;
	protected ListViewer modelFileListViewer;
	protected ListViewer aliasListViewer;
	protected ListViewer rulesListViewer;
	protected Combo typeCombo;
	protected Composite documentDetailComposite;
	protected Text nameText;
	protected Text baseURIText;
	protected Text descriptionText;
	protected Composite ruleAddRemoveButtons;

	public ExportToSMLIFWizardPage2(String pageName, IStructuredSelection selection) {
		super(pageName);
		setTitle(pageName); //NON-NLS-1
		//setDescription("Create an SML-IF file from the SML files available in the selected project."); //NON-NLS-1
		
		setSelection(selection);
	}

	protected void setSelection(IStructuredSelection selection) {
		this.selection = selection;
	}
	
	protected void updateDocumentList() {
		//modelFileListViewer.refresh();
		modelFileListViewer.setInput(getModel());
	}

	public void createControl(Composite parent) {
		Composite topComponent = new Composite(parent, SWT.NONE);
		GridData topComponentData = new GridData(GridData.GRAB_HORIZONTAL
				| GridData.FILL_HORIZONTAL);
		topComponent.setLayoutData(topComponentData);

		GridLayout topComponentLayout = new GridLayout();
		topComponentLayout.numColumns = 1;
		topComponentLayout.makeColumnsEqualWidth = false;
		topComponent.setLayout(topComponentLayout);
		
		createDocumentInformationGroup(topComponent);
		createIdentityInformationGroup(topComponent);

		setControl(topComponent);

		setPageComplete(true);
		Dialog.applyDialogFont(topComponent);
		//SwtDebugUtility.getInstance().decorate(topComponent);
	}

	protected void createIdentityInformationGroup(Composite topComponent) {
		Group identityInformationGroup = new Group(topComponent, SWT.NONE);
		identityInformationGroup.setText(SMLMessages.exportWizardIdentityInformationGroupTitle);
		GridData identityInformationGroupData = new GridData(GridData.GRAB_HORIZONTAL
				| GridData.FILL_BOTH);
		identityInformationGroupData.horizontalSpan = 1;
		identityInformationGroup.setLayoutData(identityInformationGroupData);
	
		GridLayout documentInformationGroupLayout = new GridLayout();
		documentInformationGroupLayout.numColumns = 2;
		documentInformationGroupLayout.makeColumnsEqualWidth = true;
		identityInformationGroup.setLayout(documentInformationGroupLayout);
		
		createNameAndURIComposite(identityInformationGroup);
		createDescriptionComposite(identityInformationGroup);
	}
	
	protected void createDescriptionComposite(Group identityInformationGroup) {
		Composite descriptionComposite = new Composite(identityInformationGroup, SWT.NONE);
		GridData gd = new GridData(GridData.FILL_BOTH);
		//gd.horizontalSpan = GridData.FILL;
		descriptionComposite.setLayoutData(gd);
		
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.makeColumnsEqualWidth = false;
		descriptionComposite.setLayout(layout);
		
		createFormLabel(descriptionComposite, SMLMessages.exportWizardDescriptionLabel);

		gd = new GridData();
		gd.horizontalSpan = 1;
		gd.horizontalAlignment = SWT.FILL;
		gd.verticalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = true;

		descriptionText = new Text(descriptionComposite, SWT.MULTI | SWT.BORDER);
		descriptionText.setLayoutData(gd);
	}

	protected void createNameAndURIComposite(Group identityInformationGroup) {
		Composite nameAndURIComposite = new Composite(identityInformationGroup, SWT.NONE);
		GridData gd = new GridData(GridData.FILL_BOTH);
		//gd.horizontalSpan = GridData.FILL;
		nameAndURIComposite.setLayoutData(gd);
		
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.makeColumnsEqualWidth = false;
		nameAndURIComposite.setLayout(layout);
		
		createFormLabel(nameAndURIComposite, SMLMessages.exportWizardNameLabel);

		gd = new GridData();
		gd.horizontalSpan = 1;
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;

		nameText = new Text(nameAndURIComposite, SWT.SINGLE | SWT.BORDER);
		nameText.setLayoutData(gd);

		createFormLabel(nameAndURIComposite, SMLMessages.exportWizardBaseURILabel);

		baseURIText = new Text(nameAndURIComposite, SWT.SINGLE | SWT.BORDER);
		baseURIText.setLayoutData(gd);
	}

	protected void createDetectedDocumentsLabel(Composite topComponent) {
		Label detectedDocumentsLabel = new Label(topComponent, SWT.NONE);
		detectedDocumentsLabel.setText(SMLMessages.exportWizardDetectedDocumentsLabel);
		GridData detectedDocumentsLabelData = new GridData(GridData.GRAB_HORIZONTAL
				| GridData.FILL_HORIZONTAL);
		detectedDocumentsLabelData.grabExcessHorizontalSpace = true;
		detectedDocumentsLabelData.horizontalSpan = 2;
		detectedDocumentsLabel.setLayoutData(detectedDocumentsLabelData);
	}


	protected void createDocumentInformationGroup(Composite topComponent) {
		Group documentInformationGroup = new Group(topComponent, SWT.NONE);
		documentInformationGroup.setText(SMLMessages.exportWizardDocumentInformationGroupLabel);
		GridData documentInformationGroupData = new GridData(GridData.GRAB_HORIZONTAL
				| GridData.FILL_BOTH);
		documentInformationGroupData.horizontalSpan = 1;
		documentInformationGroup.setLayoutData(documentInformationGroupData);
	
		GridLayout documentInformationGroupLayout = new GridLayout();
		documentInformationGroupLayout.numColumns = 2;
		documentInformationGroupLayout.makeColumnsEqualWidth = true;
		//destinationGroupLayout.marginLeft = 3;
		//destinationGroupLayout.marginRight = 3;
		documentInformationGroup.setLayout(documentInformationGroupLayout);
		
		createDetectedDocumentsLabel(documentInformationGroup);
		createDocumentList(documentInformationGroup);
		createDocumentDetailComposite(documentInformationGroup);
	}

	protected void createDocumentDetailComposite(Group documentInformationGroup) {
		documentDetailComposite = new Composite(documentInformationGroup, SWT.NONE);
		GridData gd = new GridData(GridData.FILL_BOTH);
		//gd.horizontalSpan = GridData.FILL;
		documentDetailComposite.setLayoutData(gd);
		documentDetailComposite.setEnabled(false);
		
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		layout.makeColumnsEqualWidth = false;
		documentDetailComposite.setLayout(layout);
		
		createTypeControls(documentDetailComposite);
		createAliasControls(documentDetailComposite);
		createRulesControls(documentDetailComposite);
	}

	protected void createRulesControls(Composite parent) {
		createFormLabel(parent, SMLMessages.exportWizardRulesLabel);
		
		rulesListViewer = createStringListViewer(parent);
		
		//addDefaultBehaviorMessage();

		createRuleAddRemoveButtons(parent);
	}

	protected void createAliasControls(Composite parent) {
		createFormLabel(parent, SMLMessages.exportWizardAliasesLabel);
		
		aliasListViewer = createStringListViewer(parent);
		
		//addDefaultBehaviorMessage();

		createAliasAddRemoveButtons(parent);
	}

	protected void createFormLabel(Composite parent, String labelText) {
		Label aliasLabel = new Label(parent, SWT.RIGHT);
		aliasLabel.setText(labelText);
		GridData gd = new GridData();
		gd.horizontalSpan = 1;
		gd.verticalAlignment = SWT.BEGINNING;
		gd.horizontalAlignment = SWT.FILL;
		aliasLabel.setLayoutData(gd);
	}

	protected ListViewer createStringListViewer(Composite parent) {
		GridData gd;
		ListViewer listViewer = new ListViewer(parent);
		gd = new GridData();
		gd.horizontalSpan = 1;
		gd.horizontalAlignment = SWT.FILL;
		gd.verticalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		listViewer.getControl().setLayoutData(gd);
		listViewer.setLabelProvider(new LabelProvider() {
			public String getText(Object element) {
				return (String) element;
			}
		});
		listViewer.setContentProvider(new ArrayContentProvider());
		return listViewer;
	}

	protected void createAliasAddRemoveButtons(Composite parent) {
		Composite buttonComposite = createVerticalButtonPanel(parent);
		
		Button addButton = createVerticalButtonPanelButton(buttonComposite, SMLMessages.exportWizardAddButtonLabel);
		addButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				IInputValidator validator = new IInputValidator() {
					public String isValid(String newText) {
						if ("".equals(newText.trim())) {
							return null;
						}
						boolean valid = false;
						try {
							valid = new URI(newText).getPath().equals(newText);
						} catch (URISyntaxException e) {
						}
						if (valid) {
							return null;
						}
						return NLS.bind(SMLMessages.exportWizardErrorURINotValid, newText);
					}
				};
				InputDialog input = new InputDialog(getShell(), SMLMessages.exportWizardNewAliasTitle, SMLMessages.exportWizardNewAliasPrompt, "", validator); //$NON-NLS-3$ //$NON-NLS-1$ //$NON-NLS-1$ //$NON-NLS-1$ //$NON-NLS-1$
				if (input.open() == InputDialog.OK) {
					getSelectedModelUnitFile().addAlias(input.getValue().trim());
					ruleAddRemoveButtons.setEnabled(true);
					aliasListViewer.refresh();
				}
			}
		});

		Button removeButton = createVerticalButtonPanelButton(buttonComposite, SMLMessages.exportWizardRemoveButtonText);
		removeButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				int[] selectedAliases = aliasListViewer.getList().getSelectionIndices();
				for (int i = 0; i < selectedAliases.length; i++) {
					getSelectedModelUnitFile().removeAlias(selectedAliases[i]);
					aliasListViewer.refresh();
					if (aliasListViewer.getList().getItemCount() == 0) {
						ruleAddRemoveButtons.setEnabled(false);
					}
				}
			}
		});
	}

	protected Composite createVerticalButtonPanel(Composite parent) {
		Composite buttonComposite = new Composite(parent, SWT.NONE);
		GridLayout buttonLayout = new GridLayout();
		buttonLayout.numColumns = 1;
		buttonLayout.marginHeight = 0;
		buttonLayout.marginWidth = 0;
		buttonComposite.setLayout(buttonLayout);
		GridData buttonCompositeData = new GridData();
		buttonCompositeData.verticalAlignment = GridData.FILL;
		buttonComposite.setLayoutData(buttonCompositeData);
		return buttonComposite;
	}

	protected void createRuleAddRemoveButtons(Composite parent) {
		ruleAddRemoveButtons = createVerticalButtonPanel(parent);
		Button addButton = createVerticalButtonPanelButton(ruleAddRemoveButtons, SMLMessages.exportWizardAddButtonLabel);
		addButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				ListDialog dialog = new ListDialog(getShell());
				dialog.setTitle(SMLMessages.exportWizardNewRuleDialogTitle);
				dialog.setMessage(SMLMessages.exportWizardNewRuleDialogPrompt);
				List choices = getRuleListChoices();
				dialog.setLabelProvider(new LabelProvider());
				dialog.setContentProvider(new ArrayContentProvider());
				dialog.setInput(choices);
//				InputDialog input = new InputDialog(getShell(), SMLMessages.exportWizardNewRuleDialogTitle, SMLMessages.exportWizardNewRuleDialogPrompt, "", validator); //$NON-NLS-3$ //$NON-NLS-1$ //$NON-NLS-1$ //$NON-NLS-2$
				if (dialog.open() == InputDialog.OK) {
					Object[] selections = dialog.getResult();
					for (int i = 0; i < selections.length; i++) {
						getSelectedModelUnitFile().addRuleForBindings((String) selections[i]);
					}
					rulesListViewer.refresh();
				}
			}
		});

		Button removeButton = createVerticalButtonPanelButton(ruleAddRemoveButtons, SMLMessages.exportWizardRemoveButtonText);
		removeButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				int[] selectedRules = aliasListViewer.getList().getSelectionIndices();
				for (int i = 0; i < selectedRules.length; i++) {
					getSelectedModelUnitFile().removeRuleForBindings(selectedRules[i]);
					rulesListViewer.refresh();
				}
			}
		});
	}

	protected List getRuleListChoices() {
		// TODO build this list by parsing .xsd files in the document list for schematron rules
		return Arrays.asList(new String[]{"apple", "banana", "orange"});
	}

	protected Button createVerticalButtonPanelButton(Composite buttonComposite, String buttonText) {
		Button button = new Button(buttonComposite, SWT.NONE);
		button.setText(buttonText);
		GridData buttonData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		buttonData.verticalAlignment = SWT.BEGINNING;
		button.setLayoutData(buttonData);
		return button;
	}

	protected void createTypeControls(Composite parent) {
//		Composite typeComposite = new Composite(documentInformationGroup, SWT.NONE);
//		GridData typeCompositeData = new GridData(GridData.GRAB_HORIZONTAL
//				| GridData.FILL_HORIZONTAL);
//		typeComposite.setLayoutData(typeCompositeData);
		
		Label typeLabel = new Label(parent, SWT.RIGHT);
		typeLabel.setText(SMLMessages.exportWizardTypeLabel);
		GridData gd = new GridData();
		gd.horizontalSpan = 1;
		gd.horizontalAlignment = SWT.FILL;
		typeLabel.setLayoutData(gd);
		
		typeCombo = new Combo(parent, SWT.READ_ONLY);
		typeCombo.add(SMLMessages.exportWizardTypeComboGenic);
		typeCombo.add(SMLMessages.exportWizardTypeComboPhenic);
		typeCombo.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				if (typeCombo.getText().equals(SMLMessages.exportWizardTypeComboGenic)) {
					getSelectedModelUnitFile().setType(ISMLConstants.TYPE_GENIC);
				}
				if (typeCombo.getText().equals(SMLMessages.exportWizardTypeComboPhenic)) {
					getSelectedModelUnitFile().setType(ISMLConstants.TYPE_PHENIC);
				}
			}
		});
		gd = new GridData();
		gd.horizontalSpan = 1;
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		typeCombo.setLayoutData(gd);
		
		Label spacer = new Label(parent, SWT.NONE);
		gd = new GridData();
		gd.horizontalSpan = 1;
		gd.horizontalAlignment = SWT.FILL;
		spacer.setLayoutData(gd);
	}

	protected void createDocumentList(Group documentInformationGroup) {
		modelFileListViewer = new ListViewer(documentInformationGroup, SWT.SINGLE | SWT.BORDER);
		modelFileListViewer.setContentProvider(new ArrayContentProvider() {
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
				super.inputChanged(viewer, oldInput, newInput);
			}
		});
		modelFileListViewer.setLabelProvider(new LabelProvider() {
			public String getText(Object element) {
				return ((ModelUnitFile) element).getFileName();
			}
		});
		modelFileListViewer.setSorter(new ViewerSorter() {
			public int compare(Viewer viewer, Object resource1, Object resource2) {
				return ((ModelUnitFile) resource1).getFileName().compareTo(((ModelUnitFile) resource2).getFileName());
			}
		});
		modelFileListViewer.setInput(getModel());
		modelFileListViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				if (event.getSelection().isEmpty()) {
					resetDocumentDetailComposite();
				} else {
					documentDetailComposite.setEnabled(true);
					ModelUnitFile selectedModelUnitFile = (ModelUnitFile) ((IStructuredSelection) event.getSelection()).getFirstElement();
					aliasListViewer.setInput(selectedModelUnitFile.getAliases());
					rulesListViewer.setInput(selectedModelUnitFile.getRulesForBindings());
					if (selectedModelUnitFile.getAliases().size() == 0) {
						ruleAddRemoveButtons.setEnabled(false);
					}
					typeCombo.select((selectedModelUnitFile.getType() == ISMLConstants.TYPE_GENIC) ? typeCombo.indexOf(SMLMessages.exportWizardTypeComboGenic) : typeCombo.indexOf(SMLMessages.exportWizardTypeComboPhenic));
				}
			}
		});
		
		GridData listViewerData = new GridData(GridData.FILL_BOTH);
		modelFileListViewer.getControl().setLayoutData(listViewerData);
	}

	protected Object[] getModel() {
		return ((ExportToSMLIFWizard) getWizard()).getSelectedFiles().values().toArray();
	}

	/*
	 * Clean up document detail composite when no document
	 * is selected
	 */
	protected void resetDocumentDetailComposite() {
		documentDetailComposite.setEnabled(false);
		aliasListViewer.setInput(new Object[]{});
		rulesListViewer.setInput(new Object[]{});
		typeCombo.deselectAll();
	}

	public String getSMLIFName() {
		return nameText.getText();
	}

	public String getBaseURI() {
		return baseURIText.getText();
	}

	public String getSMLIFDescription() {
		return descriptionText.getText();
	}

	protected ModelUnitFile getSelectedModelUnitFile() {
		return (ModelUnitFile) ((IStructuredSelection) modelFileListViewer.getSelection()).getFirstElement();
	}

}
