/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.export.ui;

import java.util.ArrayList;

import org.eclipse.core.resources.IFile;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLCommonUtil;

public class ModelUnitFile {
	private static final int UNINITIALIZED_INT = -1;

	protected IFile file;
	protected ArrayList aliases = new ArrayList();
	protected ArrayList rulesForBindings = new ArrayList();
	protected ArrayList discoveredSchematronRules = null;
	protected int type = UNINITIALIZED_INT;

	public ModelUnitFile(IFile file) {
		super();
		this.file = file;
	}
	
	public String getFileName() {
		return file.getName();
	}

	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + ((getFileName() == null) ? 0 : getFileName().hashCode());
		return result;
	}

	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof ModelUnitFile)) {
			return false;
		}
		final ModelUnitFile other = (ModelUnitFile) obj;
		if (getFileName() == null) {
			if (other.getFileName() != null) {
				return false;
			}
		} else if (!getFileName().equals(other.getFileName())) {
			return false;
		}
		return true;
	}

	public int getType() {
		if (type == UNINITIALIZED_INT) {
			type = SMLCommonUtil.identifyDocumentType(file);
		}
		return type;
	}

	public ArrayList getAliases() {
		return aliases;
	}

	public ArrayList getRulesForBindings() {
		return rulesForBindings;
	}

	public void setType(int newType) {
		this.type = newType;
	}

	public boolean addAlias(String string) {
		return aliases.add(string);
	}

	public boolean removeAlias(Object string) {
		return aliases.remove(string);
	}

	public void removeAlias(int index) {
		aliases.remove(index);
	}

	public boolean addRuleForBindings(String string) {
		return rulesForBindings.add(string);
	}
	
	public void removeRuleForBindings(int index) {
		rulesForBindings.remove(index);
	}

	public IFile getFile() {
		return file;
	}

	public boolean addDiscoveredSchematronRule(String string) {
		if (discoveredSchematronRules == null) {
			initDiscoveredSchematronRules();
		}
		return discoveredSchematronRules.add(string);
	}

	public void initDiscoveredSchematronRules() {
		discoveredSchematronRules = new ArrayList();
	}
	
	public ArrayList getDiscoveredSchematronRules() {
		return discoveredSchematronRules;
	}

}
