/**********************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.importing;

import java.io.File;
import java.io.IOException;

import org.eclipse.cosmos.rm.smlif.internal.common.SMLConstants;


/**
 * This represents an individual genic or phenic document
 * 
 * @author David Whiteman
 */
public class ModelUnit {
	public static final int GENIC = 0;
	public static final int PHENIC = 1;
	
	private StringBuffer fileNameBuffer = new StringBuffer();
	private StringBuffer contents = new StringBuffer();
	private int type;

	public ModelUnit(int type) {
		this.type = type;
	}

	public void start() {
		contents.append(SMLConstants.XML_PREAMBLE).append(SMLConstants.nl);
	}

	public void stop() {
//		System.out.println(contents.toString());
//		System.out.println("======================");
//		System.out.println(" END OF FILE "+getFileName());
//		System.out.println("======================");
	}

	public StringBuffer getFileContents() {
		return contents;
	}

	public String getFileName() {
		String fileName = fileNameBuffer.toString().trim();
		if ("".equals(fileName)) {
			File tempFile = null;
			try {
				// Generate random name for file if no alias provided
				// TODO better way to handle no alias???
				// Maybe a file name based on SML-IF's model name?
				tempFile = File.createTempFile("temp", getFileSuffix());
			} catch (IOException e) {
			}
			fileName = tempFile.getName();
		}
		return fileName;
	}

	private String getFileSuffix() {
		return (type == GENIC) ? ".xsd" : ".xml";
	}

	public void appendToFileName(String fileNameSegment) {
		fileNameBuffer.append(fileNameSegment);
	}

	public void appendToContents(String string) {
		contents.append(string);
	}
}
