/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.importing.ui;

import java.io.File;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLMessages;
import org.eclipse.cosmos.rm.smlif.internal.dialogs.ResourceSelectionDialog;
import org.eclipse.cosmos.rm.smlif.internal.importing.ImportException;
import org.eclipse.cosmos.rm.smlif.internal.importing.ImportFromSMLIF;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;


public class ImportFromSMLIFWizardPage extends WizardPage {
	
	protected IStructuredSelection selection;

	protected Text modelDirText;
	protected Text smlifFileText;
	protected Button importFromWorkspaceButton;
	protected Button importFromFileSystemButton;
	protected Button createFolderStructureButton;
	protected Button genMetadataFileButton;
	protected Button overwriteFilesWithoutWarningButton;

	public ImportFromSMLIFWizardPage(String pageName, IStructuredSelection selection) {
		super(pageName);
		setTitle(pageName);
		
		setSelection(selection);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.WizardNewFileCreationPage#createAdvancedControls(org.eclipse.swt.widgets.Composite)
	 */	
	public void createControl(Composite parent) {
		Composite topComposite = new Composite(parent, SWT.NONE); 

		GridLayout topCompositeLayout = new GridLayout();
		topCompositeLayout.numColumns = 1;
		topCompositeLayout.makeColumnsEqualWidth = false;
		
		topComposite.setLayout(topCompositeLayout);
		
		Group pathsGroup = createPathsGroup(topComposite);
		createSourceFileSection(pathsGroup);
		createDestinationLocationSection(pathsGroup);
		createCheckBoxOptions(topComposite);
		
		setControl(topComposite);//(IResource) selection.getFirstElement()
		String dirNameSuggestion = null;
		if ((selection == null) || (selection.size() == 0)) {
			dirNameSuggestion = "/";
		} else {
			dirNameSuggestion = ((IFile) selection.getFirstElement()).getParent().getFullPath().toString();
		}
		modelDirText.setText(dirNameSuggestion);
		smlifFileText.setText(getInitialSourceFileName());
		updatePageComplete();
		Dialog.applyDialogFont(topComposite);
	}

	protected String getInitialSourceFileName() {
		if (selection == null || selection.isEmpty()) {
			return "";
		}
		IResource sel = (IResource) selection.getFirstElement();
		return sel.getFullPath().toString();
	}

	protected void createCheckBoxOptions(Composite topComposite) {
		genMetadataFileButton = new Button(topComposite, SWT.CHECK);
		genMetadataFileButton.setText(SMLMessages.importWizardOptionGenerateMetadata);
		genMetadataFileButton.setSelection(true);
		genMetadataFileButton.setEnabled(false);
		
		overwriteFilesWithoutWarningButton = new Button(topComposite, SWT.CHECK);
		overwriteFilesWithoutWarningButton.setText(SMLMessages.importWizardOptionOverwriteFiles);
		overwriteFilesWithoutWarningButton.setEnabled(false);
		
		createFolderStructureButton = new Button(topComposite, SWT.CHECK);
		createFolderStructureButton.setText(SMLMessages.importWizardOptionCreateStructure);
		createFolderStructureButton.setSelection(true);
	}

	protected Group createPathsGroup(Composite topComposite) {
		Group pathsGroup = new Group(topComposite, SWT.NONE);
		pathsGroup.setText(SMLMessages.importWizardPathsGroupTitle);
		GridData pathsGroupData = new GridData(GridData.GRAB_HORIZONTAL
				| GridData.FILL_HORIZONTAL);
		pathsGroup.setLayoutData(pathsGroupData);

		GridLayout pathsGroupLayout = new GridLayout();
		pathsGroupLayout.numColumns = 3;
		pathsGroupLayout.makeColumnsEqualWidth = false;
		pathsGroup.setLayout(pathsGroupLayout);
		return pathsGroup;
	}

	protected void createDestinationLocationSection(Group pathsGroup) {
		Label modelDirLabel = new Label(pathsGroup, SWT.NULL);
		modelDirLabel.setText(SMLMessages.importWizardLabelDestinationDirectory);
		
		modelDirText = new Text(pathsGroup, SWT.BORDER); //NON-NLS-1 //NON-NLS-2
		modelDirText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				setPageComplete(modelDirText.getText().length() > 0);
			}
		});
		modelDirText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Button browseForDirectoryButton = new Button(pathsGroup, SWT.PUSH);
		browseForDirectoryButton.setText(SMLMessages.importWizardBrowseButton);
		browseForDirectoryButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				promptForDestinationDirectory();
			}
		});
	}

	protected void createSourceFileSection(Group pathsGroup) {
		Label smlifFileLabel = new Label(pathsGroup, SWT.NULL);
		smlifFileLabel.setText(SMLMessages.importWizardLabelSourceFile);

		smlifFileText = new Text(pathsGroup, SWT.BORDER);
		smlifFileText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				updatePageComplete();
				
				// Initialize the model dir to something more
				// useful when the smlif file is changed
				// since eclipse does not respond well to
				// generating files to the workspace root dir
				if ("/".equals(modelDirText.getText())) {
					if (!"".equals(smlifFileText.getText())) {
						modelDirText.setText(new Path(smlifFileText.getText())
								.removeLastSegments(1).toString());
					}
				}
			}
		});
		GridData smlifFileData = new GridData(GridData.FILL_HORIZONTAL);
		smlifFileData.horizontalSpan = 2;
		smlifFileText.setLayoutData(smlifFileData);
		
		createBrowseButtonRow(pathsGroup);
		
	}

	protected void createBrowseButtonRow(Group pathsGroup) {
		Composite browseButtonRow = new Composite(pathsGroup, SWT.NONE);
		GridLayout browseButtonRowLayout = new GridLayout();
		browseButtonRowLayout.numColumns = 2;
		browseButtonRowLayout.marginHeight = 0;
		browseButtonRowLayout.marginWidth = 0;
		browseButtonRowLayout.makeColumnsEqualWidth = true;
		GridData gd = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gd.horizontalSpan = GridData.FILL;
		browseButtonRow.setLayoutData(gd);
		browseButtonRow.setLayout(browseButtonRowLayout);

		Button importFromWorkspaceButton = new Button(browseButtonRow, SWT.PUSH);
		importFromWorkspaceButton.setText("Browse Workspace...");
		importFromWorkspaceButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				String selectedSourceFile = promptForFileInWorkspace();
				updateSourceFileText(selectedSourceFile);
			}
		});
		Button importFromFileSystemButton = new Button(browseButtonRow, SWT.PUSH);
		importFromFileSystemButton.setText("Browse File System...");
		importFromFileSystemButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				String selectedSourceFile = promptForFileOnSystem();
				updateSourceFileText(selectedSourceFile);
			}
		});
	}

	public boolean doImport() {
		if (!checkSourceFileName()) {
			setErrorMessage(NLS.bind(SMLMessages.importWizardErrorInvalidSourceFile, smlifFileText.getText()));
			return false;
		}
		if (!checkDestinationDirName()) {
			setErrorMessage(NLS.bind(SMLMessages.importWizardErrorInvalidDestinationDirectory, modelDirText.getText()));
			return false;
		}
		
		ImportFromSMLIF importer = new ImportFromSMLIF();
		importer.setCreateNecessaryStructure(createFolderStructureButton.getSelection());
		importer.setGenerateMetadataFile(genMetadataFileButton.getSelection());
		importer.setOverwriteFilesWithoutWarning(overwriteFilesWithoutWarningButton.getSelection());
		boolean result = false;
		try {
			String fileName = smlifFileText.getText();
			IFile file = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(fileName));
			if (file != null) {
				result = importer.importFromFileSystemFile(file.getLocation().toOSString(), modelDirText.getText());
			} else {
				result = importer.importFromFileSystemFile(fileName, modelDirText.getText());				
			}
		} catch (ImportException ex) {
			SMLCommonUtil.openErrorWithDetail(SMLMessages.importErrorTitle, ex.getMessage(), ex);
		}
		return result;
	}

	/**
	 * Answer whether the destination directory entered is a valid
	 * project or folder name.  The directory does not need to
	 * exist.
	 * 
	 * @return
	 */
	protected boolean checkDestinationDirName() {
		boolean okForFolder = ResourcesPlugin.getWorkspace().validatePath(
				modelDirText.getText(), IResource.FOLDER).isOK();
		if (!okForFolder) {
			okForFolder = ResourcesPlugin.getWorkspace().validatePath(
					modelDirText.getText(), IResource.PROJECT).isOK();
		}
		return okForFolder;
	}

//	private boolean checkSourceFileName() {
//		if (sourceIsWorkspacePath()) {
//			// This answers true ONLY for valid workspace paths pointing to a file,
//			// which is exactly what we want
//			return ResourcesPlugin.getWorkspace().validatePath(
//					smlifFileText.getText(), IResource.FILE).isOK();
//		} else {
//			// isValidPath() will answer true for either c:\a\b or /a/b/c.
//			// On UNIX this is ok.  On Windows we check to see if a device
//			// is part of the path to determine if it's valid, in addition to
//			// calling isValidPath().
//			boolean deviceOK = true;
//			Path filePath = new Path(smlifFileText.getText());
//			if (Platform.getOS().equals(Platform.WS_WIN32)) {
//				deviceOK = (filePath.getDevice() != null);
//			}
//			return deviceOK && filePath.isValidPath(smlifFileText.getText());
//		}
//	}

	protected boolean checkSourceFileName() {
		// First check to see if it's a filesystem path
		File srcFile = new File(smlifFileText.getText());
		if (srcFile.exists()) {
			return true;
		}
		// If the file doesn't exist on the file system,
		// check the workspace
		Path srcPath = new Path(smlifFileText.getText());
		return ResourcesPlugin.getWorkspace().getRoot().getFile(srcPath).exists();
	}
	
	public void setSelection(IStructuredSelection selection) {
		this.selection = selection;
	}

	protected void updatePageComplete() {
		setPageComplete((smlifFileText.getText().length() > 0) && (modelDirText.getText().length() > 0));
	}

	protected void promptForDestinationDirectory() {
		ContainerSelectionDialog containerDialog = new ContainerSelectionDialog(getShell(), ResourcesPlugin.getWorkspace().getRoot(), true, SMLMessages.importWizardDestinationDirectoryDialogTitle);
		containerDialog.setMessage(SMLMessages.importWizardDestinationDirectoryDialogMessage);
		if (containerDialog.open() == Window.CANCEL) {
			return;
		}
//		if (dirDialog == null) {
//			dirDialog = new DirectoryDialog(getShell());
//			dirDialog.setText(SMLMessages.importWizardDestinationDirectoryDialogTitle);
//			dirDialog.setMessage(SMLMessages.importWizardDestinationDirectoryDialogMessage);
//			//dialog.setFilterPath(targetModelLocation);
//		}
//	
//		String name = dirDialog.open();
		String name = ((IPath) containerDialog.getResult()[0]).toString();
		modelDirText.setText(name);
	}

	protected String promptForFileInWorkspace() {
//		ResourceListSelectionDialog fileDialog = new ResourceListSelectionDialog(getShell(), ResourcesPlugin.getWorkspace().getRoot(), IResource.FILE);
//		fileDialog.setTitle(SMLMessages.importWizardSourceFileDialogTitle);
//		if (fileDialog.open() == Window.CANCEL) {
//			return null;
//		}
//		IFile name = (IFile) fileDialog.getResult()[0];
//		String relativePathToFile = name.getFullPath().toString();
//		return relativePathToFile;
		ResourceSelectionDialog resourceDialog = new ResourceSelectionDialog(getShell(), ISMLConstants.TYPE_SMLIF);
		IFile[] results = resourceDialog.open();
		if (results.length == 0) {
			return null;
		}
		return results[0].getFullPath().toString();
	}

	protected String promptForFileOnSystem() {
		FileDialog fileDialog = new FileDialog(getShell());
		fileDialog.setText(SMLMessages.importWizardSourceFileDialogTitle);
		if (smlifFileText.getText().length() > 0) {
			fileDialog.setFileName(new File(smlifFileText.getText()).getName());
			fileDialog.setFilterPath(new File(smlifFileText.getText())
					.getParentFile().getAbsolutePath());
		}
		// fileDialog.setFileName(selectedSMLIFFileName);
		// fileDialog.setFilterPath(selectedSMLIFFilePath);
		fileDialog.setFilterExtensions(new String[] { "*.xml", "*" });
		return fileDialog.open();
	}

	protected void updateSourceFileText(String relativePathToFile) {
		if (relativePathToFile == null) {
			return;
		}
		smlifFileText.setText(relativePathToFile);
	}
}
