package org.eclipse.cosmos.rm.validation.internal.databuilders;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;

import junit.framework.Test;

import org.eclipse.cosmos.rm.validation.internal.common.UtilityTester;
import org.eclipse.cosmos.rm.validation.internal.core.DatastructureBuilderImpl;
import org.eclipse.cosmos.rm.validation.internal.databuilders.IdentityConstraintDataBuilder.IdentityConstraintDecl;
import org.eclipse.cosmos.rm.validation.internal.databuilders.IdentityConstraintDataBuilder.IdentityConstraintStructure;
import org.eclipse.hyades.test.common.junit.DefaultTestArbiter;
import org.eclipse.hyades.test.common.junit.HyadesTestCase;
import org.eclipse.hyades.test.common.junit.HyadesTestSuite;
import org.xml.sax.SAXException;

/**
 * Generated code for the test suite <b>TestIdentityConstraintDataBuilder</b> located at
 * <i>/org.eclipse.cosmos.rm.validation.tests/junits/TestIdentityConstraintDataBuilder.testsuite</i>.
 */
public class TestIdentityConstraintDataBuilder extends HyadesTestCase
{
	/**
	 * Constructor for TestIdentityConstraintDataBuilder.
	 * @param name
	 */
	public TestIdentityConstraintDataBuilder(String name)
	{
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>TestIdentityConstraintDataBuilder</b> definition.
	 */
	public static Test suite()
	{
		HyadesTestSuite testIdentityConstraintDataBuilder = new HyadesTestSuite("TestIdentityConstraintDataBuilder");
		testIdentityConstraintDataBuilder.setArbiter(DefaultTestArbiter.INSTANCE).setId("F7A9917B6F69B339C929B59091DF11DB");
		testIdentityConstraintDataBuilder.addTest(new TestIdentityConstraintDataBuilder("testGenerate").setId("F7A9917B6F69B339CAE5213091DF11DB").setTestInvocationId("F7A9917B6F69B339D076E93091DF11DB"));
		testIdentityConstraintDataBuilder.addTest(new TestIdentityConstraintDataBuilder("testInValidUnique").setId("CA88AFF1D7B860F511A75560960A11DB").setTestInvocationId("CA88AFF1D7B860F520AD76C0960A11DB"));
		return testIdentityConstraintDataBuilder;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception
	{
	}

	private void generateStructure(String filePath, byte expectedConstraintType, String[] expectedFields) throws ParserConfigurationException, SAXException, IOException
	{
		IdentityConstraintDataBuilder identityDataBuilder = new IdentityConstraintDataBuilder();		
		DataBuilderRegistry.instance().registerDataStructureBuilder(IdentityConstraintDataBuilder.ID, identityDataBuilder);
				
		UtilityTester.parseDocument(this.getClass().getClassLoader().getResourceAsStream(filePath), new DatastructureBuilderImpl());
		IdentityConstraintStructure identityConstraintStruct = (IdentityConstraintStructure)identityDataBuilder.getDataStructure();
		Map declarations = identityConstraintStruct.getDeclarations();
		Map orphanedConstrainedInstances = identityConstraintStruct.getOrphanedConstrainedInstances();
		IdentityConstraintDecl[] constraints = null;
		
		assertEquals("Only one declaration was expected", 1, declarations.size());
		assertNotNull("The declaration for University was not found", (constraints = identityConstraintStruct.retrieveConstraint("urn:university", "University")));
		assertEquals("Only one declaration was expected", 1, constraints.length);
		assertEquals("The type of the declaration was expected to be a key", expectedConstraintType, constraints[0].getType());
		assertEquals("The selector of the declaration is incorrect", "smlfn:deref(tns:EnrolledStudents/tns:EnrolledStudent)", constraints[0].getSelector());
		
		List fields = constraints[0].getFields();
		assertEquals("The size of the field list is not what was expected", expectedFields.length, fields.size());
		for (int i = 0; i < expectedFields.length; i++)
		{
			assertEquals("The filed of the declaration is incorrect", expectedFields[i], (String)fields.get(i));
		}
		
		assertEquals("The number of constrained instance with alias is expected to be zero", 0, identityConstraintStruct.getConstrainedInstances().size());
		assertEquals("The number of constrained instance without an alias is expected to be one", 1, orphanedConstrainedInstances.size());
		int[] indices = (int[])((List)orphanedConstrainedInstances.get("0")).get(0);
		final int[] EXPECTED_INDICES = new int[]{0};
		
		assertEquals("The size of the indices is not what was expected", EXPECTED_INDICES.length, indices.length);
		for (int i = 0; i < EXPECTED_INDICES.length; i++)
		{
			assertEquals("The index of the constrained instance is incorrect", EXPECTED_INDICES[i], indices[i]);
		}

	}
	
	/**
	 * testGenerate
	 * @throws Exception
	 */
	public void testGenerate() throws Exception
	{
		generateStructure ("test-resources/identity/InValidKeyDuplicate.xml", IdentityConstraintDecl.KEY_TYPE, new String[]{"tns:ID"});
	}

	
	/**
	 * testInValidUnique
	 * @throws Exception
	 */
	public void testInValidUnique() throws Exception
	{
		generateStructure ("test-resources/identity/InValidUnique.xml", IdentityConstraintDecl.UNIQUE_TYPE, new String[]{"tns:Name", "tns:@SSN"});
	}

}
