package org.eclipse.cosmos.rm.validation.internal.reference;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;

import junit.framework.Test;

import org.eclipse.cosmos.rm.validation.internal.common.UtilityTester;
import org.eclipse.hyades.test.common.junit.DefaultTestArbiter;
import org.eclipse.hyades.test.common.junit.HyadesTestCase;
import org.eclipse.hyades.test.common.junit.HyadesTestSuite;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Generated code for the test suite <b>TestXPointerScheme</b> located at
 * <i>/org.eclipse.cosmos.rm.validation.tests/junits/TestXPointerScheme.testsuite</i>.
 */
public class TestXPointerScheme extends HyadesTestCase
{
	/**
	 * Constructor for TestXPointerScheme.
	 * @param name
	 */
	public TestXPointerScheme(String name)
	{
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>TestXPointerScheme</b> definition.
	 */
	public static Test suite()
	{
		HyadesTestSuite testXPointerScheme = new HyadesTestSuite("TestXPointerScheme");
		testXPointerScheme.setArbiter(DefaultTestArbiter.INSTANCE).setId("F047EAA18640A2ED702C9C808A2011DB");
		testXPointerScheme.addTest(new TestXPointerScheme("testEvaluate").setId("F047EAA18640A2ED7303D8608A2011DB").setTestInvocationId("F047EAA18640A2ED79EA82008A2011DB"));
		testXPointerScheme.addTest(new TestXPointerScheme("testBadExpression").setId("F047EAA18640A2ED5FF884A08A2F11DB").setTestInvocationId("F047EAA18640A2ED70FEBBC08A2F11DB"));
		return testXPointerScheme;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception
	{
	}

	
	/**
	 * testEvaluate
	 * @throws Exception
	 */
	public void testEvaluate() throws Exception
	{
		Document document = UtilityTester.domParseDocument(this.getClass().getClassLoader().getResourceAsStream("test-resources/references/xpointer-resource.xml"));
		XPointerScheme xpointerScheme = initialize("/Courses/Course[Name='Physics101']");
		Object result = xpointerScheme.evaluate(document);
		assertTrue("The return type is expected to be NodeList[]", result instanceof NodeList[]);
		
		NodeList nodeList = ((NodeList[])result)[0];
		assertEquals("Only one node was expected to be contained by the node list", 1, nodeList.getLength());
		assertEquals("The name of the node returned is expected to be 'Course'", "Course", nodeList.item(0).getNodeName());
	}

	/**
	 * testBadExpression
	 * @throws Exception
	 */
	public void testBadExpression() throws ParserConfigurationException, SAXException, IOException
	{
		Document document = UtilityTester.domParseDocument(this.getClass().getClassLoader().getResourceAsStream("test-resources/references/xpointer-resource.xml"));
		XPointerScheme xpointerScheme = initialize("/Courses/$#2\\3");
		try
		{
			xpointerScheme.evaluate(document);
			fail("Expected an exception to be thrown");
		} 
		catch (BadExpressionException e)
		{			
		}
		
	}

	
	private XPointerScheme initialize(String expression) throws ParserConfigurationException, SAXException, IOException
	{		
		XPointerScheme xpointerScheme = new XPointerScheme();
		xpointerScheme.setExpression(expression);
		return xpointerScheme;
	}
}
