package org.eclipse.cosmos.rm.validation.internal.reference;

import junit.framework.Test;

import org.eclipse.cosmos.rm.validation.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.validation.internal.databuilders.DataBuilderRegistry;
import org.eclipse.cosmos.rm.validation.internal.databuilders.NamespaceContextBuilder;
import org.eclipse.cosmos.rm.validation.internal.databuilders.NamespaceContextBuilder.MappedNamespaceContext;
import org.eclipse.hyades.test.common.junit.DefaultTestArbiter;
import org.eclipse.hyades.test.common.junit.HyadesTestCase;
import org.eclipse.hyades.test.common.junit.HyadesTestSuite;

/**
 * Generated code for the test suite <b>TestXmlnsScheme</b> located at
 * <i>/org.eclipse.cosmos.rm.validation.tests/junits/TestXmlnsScheme.testsuite</i>.
 */
public class TestXmlnsScheme extends HyadesTestCase
{
	/**
	 * Constructor for TestXmlnsScheme.
	 * @param name
	 */
	public TestXmlnsScheme(String name)
	{
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>TestXmlnsScheme</b> definition.
	 */
	public static Test suite()
	{
		HyadesTestSuite testXmlnsScheme = new HyadesTestSuite("TestXmlnsScheme");
		testXmlnsScheme.setArbiter(DefaultTestArbiter.INSTANCE).setId("F047EAA18640A2ED4E5C00F08A2F11DB");
		testXmlnsScheme.addTest(new TestXmlnsScheme("testEvaluate").setId("F047EAA18640A2ED560892A08A2F11DB").setTestInvocationId("F047EAA18640A2ED5B1720E08A2F11DB"));
		testXmlnsScheme.addTest(new TestXmlnsScheme("testBadExpression").setId("F047EAA18640A2ED1F7A8CB08A3011DB").setTestInvocationId("F047EAA18640A2ED255FA4D08A3011DB"));
		return testXmlnsScheme;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception
	{
	}

	/**
	 * testEvaluate
	 * @throws Exception
	 */
	public void testEvaluate() throws Exception
	{
		initialize("u = urn:university");
		MappedNamespaceContext nameSpaceContext = (MappedNamespaceContext)SMLCommonUtil.retrieveDataStructure(NamespaceContextBuilder.ID);
		
		assertEquals("The u prefix is expected to map to urn:university", "urn:university", nameSpaceContext.getNamespaceURI("u"));
		assertEquals("The urn:university URI is expected to map to the u prefix", "u", nameSpaceContext.getPrefix("urn:university"));
	}

	/**
	 * testBadExpression
	 * @throws Exception
	 */
	public void testBadExpression() 
	{
		try
		{
			initialize("u ! urn:university");
			fail("Expected an exception to be thrown");
		} 
		catch (BadExpressionException e)
		{
			assertEquals("The namespace expression, \"u ! urn:university\" is invalid.  The expected format of a namespace expression is <prefix>=<uri>", 
					e.getMessage());
		}		
	}

	
	private XmlnsScheme initialize(String expression) throws BadExpressionException
	{
		NamespaceContextBuilder nsContextBuilder = new NamespaceContextBuilder();
		DataBuilderRegistry.instance().registerDataStructureBuilder(NamespaceContextBuilder.ID, nsContextBuilder);
		
		XmlnsScheme xmlnsScheme = new XmlnsScheme();
		xmlnsScheme.setExpression(expression);
		xmlnsScheme.evaluate(null);
		return xmlnsScheme;
	}
	
	
}
