/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.core;

import java.util.Map;


/**
 * The generic interface that is shared between the different phase of 
 * the validation process.
 * 
 * @see IXMLValidator
 * @see ISMLValidator
 * @see ISchematronValidation
 * @author Ali Mehregani
 */
public interface IValidator
{
	/**
	 * This attribute indicates the environment that the validation process
	 * is being done in.  The value of this attribute can be one of the following
	 * constants:<br/>
	 * 
	 * VALUE_ENV_ECLIPSE <br/>
	 * VALUE_ENV_STANDALONE <br/>
	 * Default value is VALUE_ENV_STANDALONE
	 */
	public static final String ATTRIBUTE_ENV = "attribute.environment";
		
	/**
	 * If {@link #ATTRIBUTE_INPUT_TYPE} is set to {@value #VALUE_SML_UNIT}, then the
	 * expected value of this attribute is a set of comma separated SML unit documents; 
	 * otherwise the input is expected to be the path of an SML-IF document.
	 */
	public static final String ATTRIBUTE_INSTANCE = "validation.instance";
	
	/**
	 * Stores the class that will be used to perform the XML validation
	 */
	public static final String ATTRIBUTE_VALDIATION_XML = "attribute.validation.xml";
	
	/**
	 * Stores the classes that will be used to perform the SML (schema extension) validations
	 */
	public static final String ATTRIBUTE_VALDIATION_SML = "attribute.validation.sml";
	
	/**
	 * Stores the class that will be used to perform the schematron validation
	 */
	public static final String ATTRIBUTE_VALDIATION_SCHEMATRON = "attribute.validation.schematron";
	
	/**
	 * A path to a file which the message will be written to
	 */
	public static final String ATTRIBUTE_FILE_OUTPUT = "attribute.file.output";
	
	/**
	 * Indicates that the validation is being performed in Eclipse
	 */
	public static final String VALUE_ENV_ECLIPSE = "value.environment.eclipse";
	
	/**
	 * Indicates that the validation is being performed in standalone mode
	 */
	public static final String VALUE_ENV_STANDALONE = "value.environment.standalone";
	
	/**
	 * Indicates the  file location of the schema document.  
	 */
	public static final String ATTRIBUTE_SCHEMA = "validation.schema";
	
	/**
	 * Indicates the input type.  Valid values are:
	 * <ul>
	 * 	<li> VALUE_SML_UNIT </li>
	 *  <li> VALUE_SML_IF </li>
	 * </ul>
	 * The default value is VALUE_SML_IF
	 */
	public static final String ATTRIBUTE_INPUT_TYPE = "validation.input.type";
		
	/**
	 * Indicates the validation output attribute
	 */
	public static final String ATTRIBUTE_OUTPUT = "validation.output";
	
	/**
	 * Indicates that the input is a set of SML documents that will need to be
	 * validated.
	 */
	public static final String VALUE_SML_UNIT = "value.sml.unit";
	
	/**
	 * Indicates that the input type is an SML-IF document that will need to be
	 * validated.
	 */
	public static final String VALUE_SML_IF = "value.sml.if";

	
	/**
	 * Initialize this validation class based on the set of attributes
	 * passed in.  See the ATTRIBUTE_* available.
	 * 
	 * @param validationAttribute The validation outputs
	 */
	public void initialize(Map validationAttribute);

	
	/**
	 * Performs the validation and returns true if successful and
	 * false otherwise.
	 * 
	 * @return true if successful; false otherwise
	 */
	public boolean validate();

}
